@@ -9,7 +9,7 @@ use strict;
 use XML::STX;
 use TestHandler;
 
-print '-' x 42, "\n";
+print '-' x 47, "\n";
 print "loaded\n";
 
 my $total = 0;
@@ -17,6 +17,15 @@ my $passed = 0;
 my @failed = ();
 my @errors = (0,0,0);
 my @err0 = ();
+my $m = 0; # to measure times or not
+my $time = 0;
+
+$@ = undef;
+eval "require Time::HiRes;";
+$m = 1 unless $@;
+
+my $stx = XML::STX->new(Writer => 'XML::STX::Writer');
+$stx->{Parser} = 'XML::SAX::ExpatXS';
 
 open(INDEX,'test/_index');
 
@@ -38,7 +47,6 @@ while (<INDEX>) {
 
     my $handler = TestHandler->new();
 
-    my $stx = XML::STX->new(Writer => 'XML::STX::Writer');
     $stx->{ErrorListener} = $handler;
 
     my $transformer = $stx->new_transformer($templ_uri);
@@ -56,7 +64,10 @@ while (<INDEX>) {
 
     $transformer->{ErrorListener} = $handler;
 
+    my $t0 = Time::HiRes::time() if $m;
     $transformer->transform($source, $result);
+    my $t = Time::HiRes::time() - $t0 if $m;
+    $time += $t;
 
     $handler->{result} =~ s/\s//g;
     $ln[3] =~ s/\s//g;
@@ -64,10 +75,12 @@ while (<INDEX>) {
     #print "->$handler->{result}\n";
     #print "->$ln[3]\n";
 
-    my $dots = 40 - length($ln[0]);
+    my $dots = 35 - length($ln[0]);
 
     if ($handler->{result} eq $ln[3]) {
-	print "$ln[0]", '.' x $dots, "OK\n";
+	print "$ln[0]", '.' x $dots, "OK";
+	printf " (%.3f \s\)", $t if $m;
+	print "\n";
 	$passed++;
 
     } else {
@@ -85,7 +98,7 @@ close INDEX;
 # errors
 $total++;
 my $error_line = 'errors (' . join('-', @errors) . ')';
-my $dots = 40 - length($error_line);
+my $dots = 35 - length($error_line);
 
 if (join('-',@err0) eq join('-', @errors)) {
     print $error_line, '.' x $dots, "OK\n";
@@ -95,14 +108,15 @@ if (join('-',@err0) eq join('-', @errors)) {
     print $error_line, '.' x $dots, "FAILED\n";
     push @failed, 'errors';
 }
-print '-' x 42, "\n";
+print '-' x 47, "\n";
 
 if ($passed == $total) {
     print "All tests passed successfully: $passed/$total\n";
+    printf "Total time: %.3f s\n", $time if $m;
 
 } else {
     print "There were problems: $passed/$total\n";
     print '(', join(', ', @failed), ")\n";
 }
 
-print '-' x 42, "\n";
+print '-' x 47, "\n";
